/*
 * radbf.c A tool for blowfish encryption, decryption and
 *      for Base64 encode/decode	
 *
 * Version:	$Id$
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 * Copyright 2007 ARNES <aaa-podpora@arnes.si>, http://www.arnes.si
 */

static const char rcsid[] = "$Id$";

#include <freeradius-devel/autoconf.h>
#include <openssl/rand.h>
#include <openssl/evp.h>
#include <string.h>
#include <unistd.h>

#include <freeradius-devel/libradius.h>
#include <freeradius-devel/base64.h>
#include <freeradius-devel/blowfish.h>

static const char fr_blowfish_magic[] = "Salted__";

/* =================================================================================== */
static void NEVER_RETURNS usage(char *prog)
{
	fprintf(stderr, "Usage:\n");
	fprintf(stderr, "%s [-e|-d] <master_password> <text>\n", prog);
	fprintf(stderr, "%s [-e|-d] [-b] <text>\n", prog);
	fprintf(stderr, "\t-e - encrypt\n");
	fprintf(stderr, "\t-d - decrypt\n");
	fprintf(stderr, "\t<master_password> - secret to encrypt or decrypt the text with\n");
	fprintf(stderr, "\t<text> - text to encrypt or decrypt\n");
	fprintf(stderr, "\tText will be encrypted or decrypted using a Blowfish CFB cipher.\n");
	fprintf(stderr, "\tAlternatively you can use \'openssl bf-cfb -d -a -in infile\' to decode\n");
	fprintf(stderr, "\t-b - base64 encode/decode a string\n");

	exit(1);
}
/* =================================================================================== */

int main(int argc, char *argv[]) {
	const char *pass;
	char *from;
	char *to;
	int encrypt = 0;
	int decrypt = 0;
	int base64 = 0;
	unsigned int tmp;
	char ich;
	
	while (-1 != (ich = getopt(argc, argv, "bde"))) {
		switch (ich) {
			case 'e':
				encrypt = 1;
				break;
			case 'd':
				decrypt = 1;
				break;
			case 'b':
				base64 = 1;
				break;
		}
	}
	// XOR. Only encode or only decode.
	if (1 != encrypt + decrypt) {
		usage(argv[0]);
		return -1;
	}

	if (base64) {
		if (1 != argc - optind) {
			usage(argv[0]);
			return -1;
		}
		from = argv[optind];
		if (encrypt) {
			fr_base64_encode_alloc(from, strlen(from), &to);
		}
		if (decrypt) {
			fr_base64_decode_alloc(from, strlen(from), &to, &tmp);
			if (to) to[tmp] = '\0';
		}
	} else { // not base64
	
		if (2 != argc - optind) {
			usage(argv[0]);
			return -1;
		}

		pass = argv[optind++];
		from = argv[optind];

		if (encrypt) to = fr_blowfish_encrypt(pass, from);
		if (decrypt) to = fr_blowfish_decrypt(pass, from);
	}

	if (!to) {
		fprintf(stderr, "ERROR: Encryption/decryption process failed!\n");
		return -1;
	}
	printf("%s\n", to);
	free(to);
	return 0;

}
